function pdf=fftnoncentralchi2(z,df,noncen)
% Uses the FFT to compute the pdf, at z, of the noncentral chi^2(df,noncen).

% set the tuning parameters for the FFT
pmax=18; step=0.01; p=14; 
maxz=round(max(abs(z)))+5;
while((maxz/step+1)>2^(p-1))
  p=p+1; 
end;
if p > pmax, p=pmax; end
if maxz/step+1 >2^(p-1)
  step=(maxz+1)*1.001/(2^(p-1));
end;

% compute the pdf on a grid of points
zs=sort(z); [xgrd,bigpdf]=runthefft(p,step, df,noncen);
% Now use linear interpolation to get the pdf at the desired values.
pdf=wintp1(xgrd,bigpdf,zs);

function [x,p]=runthefft(n,h, df,noncen)
n=2^n; x=(0:n-1)'*h-n*h/2; s=1/(h*n);
t=2*pi*s*((0:n-1)'-n/2);
sgn=ones(n,1); sgn(2:2:n)=-1*ones(n/2,1);
CF = (1-2*i*t).^(-df/2) .* exp((i*t*noncen)./(1-2*i*t));
% CF is the characteristic function in question; 
%   in this case it is for the noncentral chi^2.
phi=sgn.*CF; phi(n/2+1)=sgn(n/2+1); p=s.*abs(fft(phi)); 

