function [pdf,cdf] = ncbeta(bvec,n1,n2,theta1,theta2,useSPA)
% [pdf,cdf] = ncbeta(bvec,n1,n2,theta1,theta2,useSPA)
%
% Evaluates the pdf and, optionally, the cdf of the noncentral Beta distribution
%   at the ordinates in bvec
% if useSPA=1, then use the saddlepoint approximation to the noncentral F
% otherwise use exact calculation (DEFAULT)

n=n1/n2; xvec=( bvec./(1-bvec) ) / n; dxdb = (1-bvec).^(-2) / n;
acclevel=1; % just relevant for the SPA

if useSPA==1
  pdf = SPncfpdf(xvec,2*n1,2*n2,theta1,theta2,acclevel) .* dxdb;
else
  pdf =   ncfpdf(xvec,2*n1,2*n2,theta1,theta2)' .* dxdb;
end

if nargout>1
  if useSPA==1
    cdf = SPncf(xvec,2*n1,2*n2,theta1,theta2,acclevel);
  else
    cdf = ncf(xvec,2*n1,2*n2,theta1,theta2);
  end
end    
