function [pdf,cdf] = nctspa(a,n,mu,theta,renorm,rec);
% [pdf,cdf] = nctspa(a,n,mu,theta,renorm);
% SPA to the pdf and cdf of a doubly noncentral t r.v.
% with degrees of freedom n and numerator and demominator
% noncentrality parameters mu and theta, resp.
% pass renorm=1 to renormalize the SPA to the pdf.
% There is a last argument called rec. DO NOT PASS it!

if nargin<3, mu=0; end;
if nargin<4, theta=0; end;
if nargin<5, renorm=0; end;
if nargin<6, rec=0; end

alpha=mu/sqrt((1+theta/n)); normconst=1;
if renorm==1 & rec==0
  term1 = quadl(@nctspa,alpha-1e+10,alpha,[],[],n,mu,theta);
  term2 = quadl(@nctspa,alpha,alpha+1e+10,[],[],n,mu,theta);
  normconst=1/(term1+term2);
end    
cdf=zeros(size(a));pdf=cdf;
c3=n^2+2*n.*a.^2+a.^4;c2=(-2*mu*(a.^3+n.*a))./c3;
c1=(-n^2-n*a.^2-n*theta+a.^2*mu^2)./c3;c0=(n*a*mu)./c3;
q=c1/3-(c2.^2)/9;r=1/6*(c1.*c2-3.*c0)-1/27.*c2.^3;
b0=sqrt(-4*q).*cos(acos(r./sqrt(-q.^3))/3)-c2/3;
t1=-mu+a.*b0;t2=-a.*t1./b0/n/2;nu=1./(1-2*t2);
w=sqrt(-mu*t1-n*log(nu)-2*theta*nu.*t2).*sign(a-alpha);
u=sqrt((a.^2+2*n.*t2).*(2*n*nu.^2+4*theta*nu.^3)+4*n^2*b0.^2)./(2*n*b0.^2);
pdf=normconst*normpdf(w)./u;
if nargout==2
  nz=(abs(t1.*b0)>=1e-10);iz=(abs(t1.*b0)<=1e-10);
  if any(nz)
    d(nz)=1./(t1(nz).*b0(nz));
    cdf(nz)=normcdf(w(nz))+normpdf(w(nz)).*(1./w(nz)-d(nz)./u(nz));
  end
  if any(iz)
    n=sum(iz==1);
    [meull cdfiz]=nctspa([a(iz)-1e-4 a(iz)+1e-4],n,mu,theta,0,rec+1);
    if rec>5
      cdf(iz)=0.5; warning('Too many recursions');
    else
      cdf(iz)=.5*(cdfiz(1:n)+cdfiz(n+1:end));
    end
  end
end

% function [pdf,cdf] = nctspa(avec,n,mu,theta,renorm)
% % [pdf,cdf] = nctspa(avec,n,mu,theta,renorm);
% % SPA to the pdf and cdf of a doubly noncentral t r.v.
% %   with degrees of freedom n and numerator and demominator
% %   noncentrality parameters mu and theta, resp.
% % Pass renorm=1 to renormalize the SPA to the pdf.
% 
% if nargin<5,renorm=0;end
% if nargin<4,theta=0;end
% alpha=mu/sqrt((1+theta/n));
% if renorm==1
%   term1 = quadl(@nctspa,alpha-1e+10,alpha,[],[],n,mu,theta);
%   term2 = quadl(@nctspa,alpha,alpha+1e+10,[],[],n,mu,theta);
%   normconst=1/(term1+term2);
% else, normconst=1; end
% cdf=zeros(size(avec)); pdf=cdf;
% for aloop=1:length(avec);
%   a=avec(aloop);
%   c3=n^2+2*n*a^2+a^4;c2=(-2*mu*(a^3+n*a))/c3;
%   c1=(-n^2-n*a^2-n*theta+a^2*mu^2)/c3;c0=(n*a*mu)/c3;
%   q=c1/3-(c2^2)/9;r=1/6*(c1*c2-3*c0)-1/27*c2^3;
%   b0=sqrt(-4*q)*cos(acos(r/sqrt(-q^3))/3)-c2/3;
%   t1=-mu+a*b0;t2=-a*t1/b0/n/2;nu=1/(1-2*t2);
%   
%   if t1*b0 ==0
%     % a %,n,mu,theta
%     [pp1,cc1]=nctspa(a-1e-5,n,mu,theta,renorm);
%     [pp2,cc2]=nctspa(a+1e-5,n,mu,theta,renorm);
%     % observe that, in theory, one or both of the above recursive calls
%     %   could start an infinite recursion!
%     pdf(aloop) = (pp1+pp2)/2;
%     if nargout>1, cdf(aloop)=(cc1+cc2)/2; end
%   else
%     d=1/(t1*b0);
%     w=sqrt(-mu*t1-n*log(nu)-2*theta*nu*t2)*sign(a-alpha);
%     u=sqrt((a^2+2*n*t2)*(2*n*nu^2+4*theta*nu^3)+4*n^2*b0^2)/(2*n*b0^2);
%     pdf(aloop)=normconst*normpdf(w)/u;
%     if nargout>1, cdf(aloop)=normcdf(w)+normpdf(w)*(1/w-d/u); end
%   end  
% end
