function [pdf,cdf]=normlapvec(x,c,acclevel) 
% [pdf,cdf]=normlapvec(x,c,acclevel=2)
% uses SPA to compute pdf (and cdf if nargout>1) of 
%    mixture of c*normal plus (1-c)*laplace, 0<=c<=1 at x, which can be vector 
% set acclevel to 2 to use the higher order SPA.

if nargin<3, acclevel=2; end

if c==0 % pure Laplace
  pdf=exp(-abs(xvec))/2; return
elseif c==1 % pure normal
  pdf=normpdf(xvec); return
end
  
b1 = [0.98288003175962; -0.00098815145737; 0.04020574131157; ...
      0.01053174029526; 0.00580768773849;  0.67360537495647; ...
     -0.96740007771465];

b2=[19.321676; -127.31933; 351.36036; -481.0206;  326.6766;   -88.0728];
b3=[15.020794; -73.994190; 156.20436; -164.87816; 87.016516; -18.36932];

k=1-c; X2 = [1 c c^2 c^3 c^4 c^5]; X1=[X2 c^6];

if     c<=0.65,  kon=X1*b1;
elseif c<=0.85,  kon=X2*b2;
elseif c<=0.946, kon=X2*b3;
else kon=1;
end % 3rd regression gives 1 at 0.946

a2  = (-x*k^2)./(c*k)^2; a1 = -(2*k^2+c^2)/(c*k)^2; a0 = x./(c*k)^2;
q   = a1/3 -a2.^2/9; r = (a1.*a2-3*a0)/6 - a2.^3/27; m = q.^3 + r.^2;
s1  = (r+sqrt(m)).^(1/3); s2 = (r-sqrt(m)).^(1/3);
sps = s1+s2; sms = s1-s2; z3 = -sps/2 - a2/3 - sqrt(-3).*sms/2;
% z1 = sps - a2/3; z2 = -sps/2 - a2/3 + sqrt(-3).*sms/2;
s = z3;

K=0.5*s.^2.*c^2 - log(1-s.^2.*k.^2); 
Kpp=c.^2+2*k.^2 * (s.^2*k.^2+1) ./ (1-s.^2*k.^2).^2;
pdf = exp(K - s.*x)./sqrt(2*pi*Kpp);
    
if acclevel==2
  K3 = 4*s.*k.^4 .* (3+s.^2.*k.^2) ./ (1-s.^2.*k.^2).^3;
  K4 = 12*k.^4 .* (1+6*s.^2.*k.^2+s.^4.*k.^4) ./ (1-s.^2.*k.^2).^4;
  kap3= K3./(Kpp).^(3/2); kap4=K4./(Kpp).^(4/2);
  pdf = pdf .* (1 + kap4/8 - 5*(kap3.^2)/24);
  pdf = pdf / kon;
else
  pdf = pdf; % The const. of integration drops to about 0.85 as c -> 0.
end  

if nargout>1
  bad=find(abs(s)<1e-8); cdf(bad)=0.5;
  good=find(abs(s)>=1e-8); 
  x=x(good); s=s(good); 
  K=K(good); Kpp=Kpp(good);
  w=sign(s).*sqrt(2*(s.*x-K)); u=s.*sqrt(Kpp);
  npdf=normpdf(w);
  if acclevel==2
    kap3=kap3(good); kap4=kap4(good);
     O1 = -npdf.*((kap4/8-5*kap3.^2/24)./u ...
         - 1./u.^3 - kap3/2./u.^2 + 1./w.^3);
 else
    O1=0; 
  end
  cdf(good)=normcdf(w) + npdf.*(1./w - 1./u) + O1;
  
  bad=find(abs(x)<0.02); % some polishing near the mean. 
  if any(bad) 
    xx=[-0.02 0 0.02]; 
    [garb,cdf02]=normlapvec(0.02,c,acclevel);  
    yy=[1-cdf02 0.5 cdf02];
    cdf(bad) = interp1(xx',yy',x(bad));
  end

end  
