function pdf = occupancyspa(yvec,r,k)
% pdf = occupancyspa(yvec,r,k)
% SPA of the occupancy problem.
% Let Y_{k} be the number of cereal boxes one needs to purchase in order to get 
%  at least one of k different prizes, 1<=k<=r, where r is the total number of 
%  different prizes available. 
%  Then Y_{k}=Sum_{i=0}^{k-1} G_{i}, where G_{i} ~ Geo(p_{i}).

ivec=0:(k-1); pvec=(r-ivec)/r;

yl=length(yvec); pdf=zeros(yl,1);
for loop=1:yl
  y=yvec(loop); shat = occspe(y,pvec);
  qet=(1-pvec)*exp(shat);
  Ks=k*shat+sum(log(pvec)) - sum(log(1-qet));
  Kpp = sum(qet./(1-qet).^2);
  pdf(loop) = exp(Ks-y*shat) / sqrt(2*pi*Kpp);  
end  


function spe=occspe(y,pvec)
q=1-pvec; opt=optimset('Display','none','TolX',1e-6);
uplim = -log(max(q)); lolim = -1e1;

%tvec=-1:0.001:uplim; vv=zeros(length(tvec),1); 
%for tl=1:length(tvec), t=tvec(tl); vv(tl) = speeq(t,q,y); end
%plot(tvec,vv)

spe=fzero(@speeq,[lolim,0.9999*uplim],opt,q,y);

function dd=speeq(t,q,y)
k=length(q); et=exp(t);
kp = k+et*sum(q./(1-q*et));
dd = y-kp;